#include "basics.h"

#include <stdarg.h>
#include <stdlib.h>

Coord	icoord_2_coord(ICoord &crd)
{
	return Coord(crd.x, crd.y);
}


void	debug(int lev, char *fmt, ...)
{
	va_list list;
	static FILE *debug_stream = stdout;

        va_start(list, fmt);
        if (lev) {
		vfprintf(debug_stream, fmt, list);
		fprintf(debug_stream, "\n");
	}
	
        va_end(list);
}

void	rg7_error(char *fmt, ...)
{
	va_list list;
	static FILE *error_stream = stdout;

        va_start(list, fmt);
        vfprintf(error_stream, fmt, list);
	fprintf(error_stream, "\n");
	
        va_end(list);

	exit(1);
}

Region	Region::sub_region(count_type ox1, count_type oy1, count_type ox2, count_type oy2)
{
	Region tmp;
	
	tmp.size.x = ox2-ox1;
	tmp.size.y = oy2-oy1;
	
	offset_point(tmp.lb, ox1, oy1);
	offset_point(tmp.rt, ox2+1, oy2+1);
	
	return tmp;
}


void	Region::offset_point(Coord &point, count_type x, count_type y)
{
	point.x = lb.x + x*resX();
	point.y = lb.y + y*resY();
}


bool	Region::contains(Region &reg, bool fully)
{
	if (fully) 
		return inside(reg.lb) && inside(reg.rt);
	
	return false;
}



std::ostream& operator <<(std::ostream &os,const Coord &obj)
{
	os << obj.x << ":" << obj.y;
	return os;
}


std::ostream& operator <<(std::ostream &os,const ICoord &obj)
{
	os << obj.x << ":" << obj.y;
	return os;
}

std::ostream& operator <<(std::ostream &os,const Region &obj)
{
	os.precision(10);
	os << "Region: " << obj.lb << " " << obj.rt << " " << obj.size.x << "," << obj.size.y <<"\n";
	return os;
}


