#ifndef	__RAS_BASICS_H
#define	__RAS_BASICS_H

// DT for geographic coordinations
typedef	double coord_type;
// DT to express geogr. distances
typedef	double dist_type;
// integer
typedef	int	count_type;
// basic data type
typedef	int dtype;

typedef	unsigned char vshort_count;

#define	VSHORT_CMAX	255

#include <string>
#include <iostream>
//#include <array>
#include <fstream>
#include <vector>
#include <list>
#include <set>
#include <map>
#include <queue>

#include <math.h>
#include <assert.h>

#include "mersennetwister.h"

// C++ string renamed
typedef	std::string rstring;

typedef std::set<rstring> String_set;

// general float
typedef	double real;

//typedef	std::vector<dtype> dtypevec;
//typedef	std::vector<dtype>::iterator dtypevec_i;

real    myrand();
real    uniform(real f, real t);
int     int_uniform(real f, real t);

void	debug(int lev, char *fmt, ...);
void	rg7_error(char *fmt, ...);
extern	unsigned int DEBUG_SETTS;


// Generic type for coordinates - in float, in integers

template<typename T>
struct	XCoord {
	XCoord() { x=y=0; }
	
	XCoord(T ex, T ey) {
		x=ex; y=ey;
	};
	
	XCoord	operator += (XCoord &k) {
		x += k.x; y += k.y;
		return *this;
	}


	XCoord	operator + (XCoord k) {
		XCoord pom(*this);
		pom.x += k.x; pom.y += k.y;
		return pom;
	}
	
	XCoord	operator * (XCoord k) {
		XCoord pom(*this);
		pom.x *= k.x; pom.y *= k.y;
		return pom;
	}
	
	XCoord	operator / (real k) {
		XCoord pom(*this);
		pom.x /= k; pom.y /= k;
		return pom;
	}
	
	XCoord	operator - (XCoord k) {
		XCoord pom(*this);
		pom.x -= k.x; pom.y -= k.y;
		return pom;
	}
	
	bool	operator == (XCoord k) {
		return x==k.x && y==k.y;
	}
	
	
	real	dist(XCoord k) {
		return sqrt((k.x-x)*(k.x-x)+(k.y-y)*(k.y-y));
	}
	
	bool	operator != (XCoord k) { return ! (*this == k); }
	
	T x, y;
};


// Geographic 2D coordinate
typedef	XCoord<coord_type> Coord;

// 2D coordinate (index to raster, size expressing)
typedef	XCoord<count_type> ICoord;

Coord	icoord_2_coord(ICoord &crd);

// Geographic region
// Defined by:
// LB - left-bottom point
// RT - right-top point
struct	Region {

	Region() {
	}
	
	// LB, RT and size
	Region(Coord elb, Coord ert, ICoord sz) {
		lb=elb; rt=ert; size=sz;
		resume();
	}
	
	Region(Coord elb, Coord ert, count_type x, count_type y) {
		lb=elb; rt=ert; size.x=x; size.y=y;
		resume();
	}
	
	Region(Region r, ICoord sz) {
		lb=r.lb; rt=r.rt; size=sz;
		resume();
	}

	bool	empty() { return totalCells()==0; }
	
	bool	inside(Coord c) {
		return (c.x >= lb.x && c.x<=rt.x && c.y>=lb.y && c.y<=rt.y);
	}
	
	bool	i_inside(ICoord c) {
		return (c.x>=0 && c.y>=0 && c.x<size.x && c.y<size.y);
	}
	
	bool	contains(Region &reg, bool fully);
	
	// computes a sub-region (used for quad-tree decomposition)
	Region	sub_region(count_type ox1, count_type oy1, count_type ox2, count_type oy2);

	void	resume() {
		saved_res.x = resX();
		saved_res.y = resY();
	}
	
	count_type countX() { return size.x; }
	count_type countY() { return size.y; }
	ICoord	countXY() { return ICoord(countX(), countY()); }
	count_type totalCells() { return countX()*countY(); }
	
	dist_type resX() { return fabs(lb.x-rt.x)/countX(); }
	dist_type resY() { return fabs(lb.y-rt.y)/countY(); }
	Coord	cell_size() { return Coord(resX(), resY()); }

	dist_type _resX() { return saved_res.x; }
	dist_type _resY() { return saved_res.y; }

	Coord	middle_point() {
		Coord crd;

		crd.x = lb.x + (rt.x - lb.x) / 2;
		crd.y = lb.y + (rt.y - lb.y) / 2;

		return crd;
	}


	Coord	point_to_coords(ICoord ic) {
		Coord pom;
		pom.x = lb.x + ic.x*resX();//+ resX()/2.0;
		pom.y = lb.y + ic.y*resY();// + resY()/2.0;
		return pom;
	}

	ICoord	coords_to_point(Coord ic) {
		ICoord pom;
		pom.x = (count_type) ((ic.x - lb.x) / resX());
		pom.y = (count_type) ((ic.y - lb.y) / resY());
		return pom;
	}

	ICoord	random_ipoint() {
		return ICoord(int_uniform(0, countX()), int_uniform(0, countY()));
	}
	
	Coord	random_gpoint() {
		return point_to_coords(random_ipoint());
	}

	ICoord	random_ipoint_from(ICoord fr) {
		return ICoord(int_uniform(0, fr.x), int_uniform(0, fr.y));
	}
	
	void	offset_point(Coord &point, count_type x, count_type y);
	
	void	print() {
		printf("Region: [%.2f, %.2f]->[%.2f,%.2f], (%d,%d), {%.5f,%.5f}\n",
		       lb.x, lb.y, rt.x, rt.y, size.x, size.y, resX(), resY());
	}
		
	Coord lb, rt, saved_res;
	ICoord	size;
//protected:
};


std::ostream& operator <<(std::ostream &os,const Region &obj);
std::ostream& operator <<(std::ostream &os,const Coord &obj);
std::ostream& operator <<(std::ostream &os,const ICoord &obj);


//	#include <boost/numeric/ublas/matrix.hpp>

enum	{
	RAS_TYPE_CHAR,
	RAS_TYPE_INT,
	RAS_TYPE_FLOAT
};


class	General_2D_buffer {
public:
	General_2D_buffer() {
		type = RAS_TYPE_INT;
		modified = false;
	}
	
	General_2D_buffer(ICoord sz, int tp=RAS_TYPE_INT) {
		type = tp;
		resize(sz);
	}
	
	int 	map(ICoord s) { return s.y*size.x + s.x; }
	
	void	resize(ICoord sz) {
		size = sz;
		//cont = new boost::numeric::ublas::matrix<dtype>;
		//cont->resize(size.x, size.y);
		cont = new dtype[size.x * size.y];
		modified = false;
	}

	~General_2D_buffer() {
		//delete cont;
		delete [] cont;
	}
	
	count_type turny(count_type y) { return size.y-y-1;}
	
	void	write(ICoord at, dtype val) {
		//(*cont)(at.x,(at.y))=val;
		cont[map(at)] = val;
		modified = true;
	}
	
	dtype	read(ICoord at) {
		//return (*cont)(at.x, (at.y));
		return cont[map(at)];
	}

	dtype	&operator() (ICoord at) {
		return cont[map(at)];
	}

	void	clear_modified() { modified=false; }
	
	void	print() {
		for (int y=0; y<size.y; y++) {
			for (int x=0; x<size.x; x++) printf("%d ", read(ICoord(x,y)));
			printf("\n");
		}
	}
	
	dtype	*buffer_address() { return cont; }
	size_t	buffer_size() { return sizeof(dtype) * size.x * size.y; }

	//boost::numeric::ublas::matrix<dtype> *cont;
	dtype	*cont;
	ICoord	size;
	int	type;
	bool	modified;
};

template<typename T>
class	Statgen {
public:
	Statgen() {
		n=0;
	}

	void	operator() (T v) {
		if (n==0) { mina=maxa=v; }
		else {
			if (v>maxa) maxa=v;
			if (v<mina) mina=v;
		}
		n++;
	}

	T	_min() { return mina;}
	T	_maxa() { return maxa;}

	long int n;
	T	mina, maxa;
};

typedef	Statgen<dtype> Stat;
typedef	Statgen<count_type> Statc;
typedef	Statgen<coord_type> Statg;


void	ras_init();
void	ras_close();


#endif	/* __RAS_BASICS_H */

