#ifndef	__COMPRESSED_BA_H
#define	__COMPRESSED_BA_H

#include <stdlib.h>
#include <string.h>

#include "global_userdefaults.h"

/*
	Compressed_ba should implement some sort of a compression.

	1) run-length encoding
	2) implement some sort of "going-through-the-raster" method (should be implemented in Ras_phtile, ...)
	
	
*/

struct	Compressed_ba {
	Compressed_ba(char *ebuf=0, size_t bfs=0) {
		
		// options: use the given buffer (usually from the ras_interface), or make your private one
		if (ebuf == 0) 
			allocate_internal_buffer();
		else {
			// a buffer for reuse (ras_interface)
			wp = buffer = ebuf;
			internal_buffer_allocated = false; 
			buf_size = bfs;
		}
		
	}
	
	~Compressed_ba() {
		if (internal_buffer_allocated)
			deallocate_internal_buffer();
		
		buffer = wp = 0;
	}
	
	void	allocate_internal_buffer();
	void	deallocate_internal_buffer();
	
	// start one (read/write) transaction
	// returns true if the buffer is ok
	bool	prepare();
	
	// assume that resulting buffer may differ from the given one
	char	*start_address();
	// size of the resulting information
	int	length();
	
	// write sz-bytes from "data"
	bool	write_n_data(void *data, size_t sz);
	bool	read_n_data(void *data, size_t sz);
	
	// cell by cell...
	bool	write_cell(dtype cell_value);
	bool	write_n_cells(dtype *cell_buf, size_t n);
	bool	read_cell(dtype &cell_value);
	
	// BA works over a given memory buffer
	char	*buffer, *wp;
	// flag
	bool	internal_buffer_allocated;
	
	size_t	buf_size, free_buf_size;
};


#endif	/* __COMPRESSED_BA_H */