#ifndef	__RAS_FILE_INTERFACE_H
#define	__RAS_FILE_INTERFACE_H

#include "basics.h"
#include "grass_raster_id.h"

#include "ras_phtile.h"
#include "ras_metadata.h"

enum	RAS_IFACE_RESPONSE {
	RAS_IFACE_TECH_FAILED=0,
	RAS_IFACE_OK=1,
	RAS_IFACE_OK_BUT_NO_DATA=2
};

#define	RAS_IFACE_GENER_OK(X)	(X==RAS_IFACE_OK || X==RAS_IFACE_OK_BUT_NO_DATA)

#define RAS_IFACE_GENER_FAIL(X)	(X==RAS_IFACE_TECH_FAILED)

/*
	Ras_file_interface defines an interface specification
	for particular implementation of a data storage.
		
	Derived classes:
		1) Ras_sqlite3_interface
		2) Ras_wms_interface
		3) Ras_grass6_interface
*/

class	Ras_file_interface {
public:
	Ras_file_interface(RG_ID *erid) {
		rid = erid;
		stat_swapR = stat_swap = stat_load = stat_test = 0;
	}
	
	virtual	~Ras_file_interface() {
	}
	
	virtual	bool	open()=0;
	virtual	bool	close()=0;
	virtual	bool	create()=0;
	virtual	bool	drop()=0;
	
		rstring	bname();
	
	virtual	bool	write_metadata()=0;
	virtual	bool	load_metadata()=0;
	
	//virtual	void	process_loaded_tile(Ras_phtile *tile)=0;
	
	virtual	void	transaction(bool begin)=0;
	
	virtual	int	swap_tile(Ras_phtile *tile)=0;
	virtual	int	load_swapped_tile(Ras_phtile *tile)=0;
	virtual	int	swapped_tile_available(Ras_phtile *tile)=0;
	
	
	virtual	ICoord	get_minimum_region() {
		return metad.minimum_region;
	}
	
	virtual	Region	get_base_region() {
		return metad.base_region;
	}
	
	virtual	dtype	get_null_value() {
		return metad.null_value;
	}
	
	
	// basic name(ID) of the file (object's original is included
	// in RG7_open_file, see RG7_system.h)
	RG_ID	*rid;
	
	// metadata object
	Ras_metadata	metad;
	
	// access statistics
	// swap requests, swap-proceeded requests, load reqs, swt_available reqs.
	int	stat_swap, stat_swapR, stat_load, stat_test;
};


#endif	/* __RAS_FILE_INTERFACE_H */

