#include "ras_phtile.h"

dtype	Ras_phtile::rlogic(ICoord coord, dtype null_value)
{
	if (!allocated())
		return null_value;
	
	return cont->read(coord);
}


void	Ras_phtile::wlogic(ICoord coord, dtype val)
{
	if (!allocated())
		return ;
	
	cont->write(coord, val);
	dirty = true;
}

ICoord	Ras_phtile::icoord_global_2_local(ICoord coord)
{
	return coord-ilb;
}
	

bool	Ras_phtile::do_allocate()
{
	cont = new General_2D_buffer(region.countXY());
	allocated_as_internal_buffer = true;
	dirty = false;
    
	return cont != 0;
}


void	Ras_phtile::do_deallocate()
{
	// check the buffer's origin
	assert(allocated_as_internal_buffer);
	
	delete cont;
	cont = 0;
}

bool	Ras_phtile::allocate_with_a_given_buffer(General_2D_buffer *buf)
{
	cont = buf;
	allocated_as_internal_buffer = false;
	//!!! remove the previous content
	
	dirty = false;
	return cont != 0;
}


General_2D_buffer *Ras_phtile::deallocate_with_a_given_buffer()
{
	// check the buffer's origin
	assert(!allocated_as_internal_buffer);
	
	General_2D_buffer *wbuf = cont;
	cont = 0;
	
	return wbuf;
}
	


void	Ras_phtile::fill_with_null_data(dtype null_value)
{
	ICoord c;
	
	assert(allocated());
	
	for (c.x=0; c.x<region.countX(); c.x++)
	for (c.y=0; c.y<region.countY(); c.y++) {
		cont->write(c, null_value);
	}
}


bool	Ras_phtile::write_to_compressed_ba(Compressed_ba &ba)
{
	if (!ba.prepare())
		return false;

	assert(allocated());

	bool res = ba.write_n_data(cont->buffer_address(), cont->buffer_size());
	
	return res;
}


bool	Ras_phtile::read_from_compressed_ba(Compressed_ba &ba)
{
	if (!ba.prepare())
		return false;

	assert(allocated());
	
	bool res = ba.read_n_data(cont->buffer_address(), cont->buffer_size());
	
	return res;
}

