#ifndef	__RAS_PHTILE_H
#define	__RAS_PHTILE_H

#include "basics.h"
#include "compressed_ba.h"


enum	PHTILE_STATUS {
	// tile has no contents
	PHTS_NEW,
	
	//
	PHTS_READY,
	PHTS_LOADED,
	PHTS_SWAPPED
};


class	Ras_phtile {
public:
	Ras_phtile(Region reg) {
		tile_xy = ICoord(0,0);
		status = PHTS_NEW;
		cont = 0;
		region = reg;
		priority = 0;
		pyramid_level = 0;
		r3d_level = 0;
	}
	
	// Read/write access to "cont"
	dtype	rlogic(ICoord coord, dtype null_value=0);
	void	wlogic(ICoord coord, dtype val);
	ICoord	icoord_global_2_local(ICoord coord);
	
	// asked when an interface does swap_tile
	bool	needs_flush() { return dirty; }
	
	// is the tile only of null-value contents?
	bool	is_cont_null() { return false; }
	
	// set all cells to a given value (null_value)
	void	fill_with_null_data(dtype null_value);
	
	// memory management - doing allocating (cont), deallocating and test
	bool	do_allocate();
	void	do_deallocate();
	
	// use a system pool of buffers
	bool	allocate_with_a_given_buffer(General_2D_buffer *buf);
	General_2D_buffer *deallocate_with_a_given_buffer();
	
	bool	allocated() { return cont!=0; }
	
	// string name
	rstring	name() { return rstring(""); }
	
	// serialize out/in the cont (called from the interface)
	bool	write_to_compressed_ba(Compressed_ba &ba);
	bool	read_from_compressed_ba(Compressed_ba &ba);
    
	void    priority_increase(int c=1) { priority+=c; }
    
	// status of the tile (see enum	PHTILE_STATUS)
	enum	PHTILE_STATUS	status;
	
	// true => tile's contents has been modified
	bool	dirty;
	
	// logic coordinate of the tile within the raster file
	// (0,0) is bottom-left 
	ICoord	tile_xy;
	
	// buffer for tile's raster contents
	General_2D_buffer *cont;
	bool	allocated_as_internal_buffer;
	
	// geographic region of the tile
	Region	region;
	
	// ICoord offset 
	ICoord	ilb;
    
	// priority in deciding tiles for deallocation
	// connected with AOI sub-system
	int     priority;
	
	// for further use
	int 	pyramid_level, r3d_level;
};

typedef	std::set<Ras_phtile *> Phtile_set;
typedef	Phtile_set::iterator Phtile_set_i;

#endif	/* __RAS_PHTILE_H */

