#ifndef	__RAS_PHYS_FILE_H
#define	__RAS_PHYS_FILE_H

#include "ras_metadata.h"
#include "ras_file_interface.h"
#include "ras_tiling.h"
#include "tile_storage.h"
#include "pool_of_phtile_buffers.h"

#include "area_of_interest.h"

// Multi-thread...
// TODO: 1) identifikovat mista pro zamky
//	2) navrhnout sytem zamku - kolik?

class	Ras_phys_file {
public:
	Ras_phys_file(Ras_file_interface &eiface) : iface(eiface), tiling(&eiface.metad), phtile_buf_pool() {
		max_available_tiles = -1;
		startup();
	}
	
	~Ras_phys_file() {
		close_all_tiles();
	}
	
	//rstring	name() { return iface.bname; }
	Region	&region() { return iface.metad.base_region; }
	
	bool	startup();
	
	// Write/Read based on a geographic coordinate
	void	wgeo(Coord coord, dtype v);
	dtype	rgeo(Coord coord);
	
	// Write/Read based on a logical coordinate
	dtype	rcell(ICoord cell_xy);
	void	wcell(ICoord cell_xy, dtype val);
		
	// deallocate all cache
	int	deallocate_all_allocated_tiles();
	int	close_all_tiles();

	// get a phtile for a given logical coordinate
	Ras_phtile *locate_tile(ICoord coord);
	// get a phtile by tile's ID
	Ras_phtile *locate_tile_xy(ICoord coord);
		
	bool	make_tile_allocated(Ras_phtile *pht);
	bool	make_tile_deallocated(Ras_phtile *pht);
	bool	deregister_tile(Ras_phtile *pht);
		
	// ensure the tile is available for read/write
	bool	rw_access_tile(Ras_phtile *pht);
		
	// memory management over tiles
	int 	number_of_allocated_tiles();
	// number_of_allocated_tiles()>max_available_tiles
	bool	needs_reallocate();
	// Alg TODO: clever alg deciding which tiles to
	// deallocate
	void	decide_n_tiles_to_deallocate(int howm_tiles, Phtile_set &chosen);
	
	// called from make_tile_allocated when
	// needs_reallocate()==true
	int	decide_and_deallocate_tiles(int howm_tiles=1);
		
	void	create_whole_raster();
    
    Ras_areas &areas() { return r_areas; }
	
/* 
---------------------------------------------------------------
	Data
---------------------------------------------------------------
*/

	// interface to the physical storage
	Ras_file_interface &iface;
	
	// object responsable for 
	// 1) translation of coords to tile_xy
	// 2) computing a region of a given tile_xy (each tile may
	// be of distinct size)
	Ras_tiling tiling;
	
	// pool of buffers
	Pool_of_Phtile_buffers phtile_buf_pool;
	
	// TODO: similarly, one may define a pool of Phtiles...
	
	// memory of tiles: map<tile_xy> -> Phtile*
	// all registered tiles (some of them are allocated)
	// allocated tiles (all are allocated)
	Tile_storage	known_tiles, allocated_tiles;

	// size of the tile cache
	int	max_available_tiles;
    
    // system of priorities on tiles
    Ras_areas   r_areas;
};

#endif	/* __RAS_PHYS_FILE_H */

