#include "ras_phys_file.h"


int 	Ras_phys_file::number_of_allocated_tiles()
{
	// this is the real number of allocated buffers
	// it may differ (>,>>) from max_available_tiles !!!
	return allocated_tiles.size();
}


bool	Ras_phys_file::needs_reallocate()
{
	// if max_available_tiles == -1  => no limits in cache
	if (max_available_tiles == -1)
		return false;
	
	return number_of_allocated_tiles() >= max_available_tiles;
}


void	Ras_phys_file::decide_n_tiles_to_deallocate(int howm_tiles, Phtile_set &chosen)
{
	// TODO: this is the expert part of the Ras_phys_file 
	// memory management
	// --> do a clever and a well-informed choice of the tile to
	// be deallocated
	// HINTS: 1) statistics of accesses to the allocated/known tiles
	//	2) priorites by AOF (areas of interest)
	
	// start with clearing the list
	chosen.clear();
	
	// in the first design&implementation, do strictly assume
	// just 1-cache-to-deallocate requests
	assert(howm_tiles == 1);
	
	// an early design ;-)
	// choose the first one in the list
	chosen.insert( allocated_tiles.begin()->second );
}


int	Ras_phys_file::decide_and_deallocate_tiles(int howm_tiles)
{
	// assuming here: 
	assert(needs_reallocate() == true);
	
	int	removed_cnt = 0;
	Phtile_set chosen_tiles;
	Phtile_set_i cht;
	
	// make the decision first...
	decide_n_tiles_to_deallocate(howm_tiles, chosen_tiles);
	
	// foreach "cht" of the chosen ones, make "cht" deallocated
	for (cht=chosen_tiles.begin(); cht!=chosen_tiles.end(); cht++) {
		
		// TODO: should check the reason of eventual failure
		if (make_tile_deallocated(*cht))
			removed_cnt++;
	}
	
	chosen_tiles.clear();
	
	// assuming here (let the upper level to check that)
	// assert(needs_reallocate() == false);
	
	// success, done
	return removed_cnt;
}

