#include "ras_sqlite3_interface.h"


Ras_sqlite3_interface::Ras_sqlite3_interface(RG_ID *erid) : Ras_file_interface(erid)
{
	sqlite3_open(bname().c_str(), &db);
	io_buf_size = USERDEFS.ba_default_size_of_buffer();
	io_buf = new char [io_buf_size];
}


Ras_sqlite3_interface::~Ras_sqlite3_interface()
{
	close();
	sqlite3_close(db);
	
	if (io_buf)
		delete [] io_buf;
}



bool	Ras_sqlite3_interface::open()
{
	return load_metadata();
}


bool	Ras_sqlite3_interface::close()
{
	printf("Stat-iface: Loads=%d, Tests=%d, Swap=%d/%d\n", 
	       stat_load, stat_test, stat_swap, stat_swapR);
    
	bool res_wm = write_metadata();
    
	return res_wm;
}


bool	Ras_sqlite3_interface::drop()
{
	int rce = sqlite3_exec(db, "drop table rg7; drop table metadata; drop table reclass;", 0, 0, 0);
	
	//!!! implement and test correctly !!!
    
	return rce == SQLITE_OK;
}


bool	Ras_sqlite3_interface::create()
{
	
	bool drop_res = drop();
	
	//if (!drop_res)
	//	return false;
	
	sqlite3_exec(db, "create table rg7(x int, y int, pyr int, r3d int, cont blob);", 0, 0, 0);
	
	sqlite3_exec(db, "create unique index rg7_ind on rg7(x, y, pyr);", 0, 0, 0);
	
	sqlite3_exec(db, "create table metadata(bminx int, bminy int, lbx DOUBLE PRECISION, lby DOUBLE PRECISION, rtx DOUBLE PRECISION, rty DOUBLE PRECISION, sx int, sy int, nullval int); ", 0, 0, 0);
	sqlite3_exec(db, "create table reclass(x1 int, x2, int, y int);", 0, 0, 0);
	
	return write_metadata();
}


int	Ras_sqlite3_interface::swap_tile(Ras_phtile *tile)
{
	sqlite3_stmt *linfo;
	int	rc, rce;
	
	stat_swap++;
	
	if (!tile->needs_flush()) {
		return RAS_IFACE_OK_BUT_NO_DATA;
	}

	if (tile->is_cont_null()) {
		//printf("Dlazdice %s plna nul\n", tile->name.c_str());
		return RAS_IFACE_OK_BUT_NO_DATA;
	}
	
	stat_swapR++;
	
	int	update = swapped_tile_available(tile);
	char	*pom = 0;

	switch (update) {
		case RAS_IFACE_TECH_FAILED:
			return RAS_IFACE_TECH_FAILED;
		case RAS_IFACE_OK:
			pom = sqlite3_mprintf("update rg7 indexed by rg7_ind set cont = ? where x=%d and y=%d and pyr=%d and r3d=%d;", tile->tile_xy.x, tile->tile_xy.y, tile->pyramid_level, tile->r3d_level);
			break;
		case RAS_IFACE_OK_BUT_NO_DATA:
			pom = sqlite3_mprintf("insert into rg7 values(%d, %d, %d, %d, ?);", tile->tile_xy.x, tile->tile_xy.y, tile->pyramid_level, tile->r3d_level);
			break;
		default:
			return RAS_IFACE_TECH_FAILED;
	}
	
	Compressed_ba cab(io_buf, io_buf_size);
	if (!tile->write_to_compressed_ba(cab))
		return RAS_IFACE_TECH_FAILED;
	
	//printf("PRik-save: %s\n", pom);
	
	rc = sqlite3_prepare(db, pom, -1, &linfo, 0);
	sqlite3_free(pom);
	
	
	if (rc == SQLITE_OK && linfo != NULL) {
		sqlite3_bind_blob(linfo, 1, cab.start_address(), cab.length(), NULL);
		rce = sqlite3_step(linfo);
		//printf("Cab length %ld\n", cab.length());
		//printf("Save rce %d\n", rce);
		rc = sqlite3_finalize(linfo);
		
		return (rce==SQLITE_DONE) ? RAS_IFACE_OK : RAS_IFACE_TECH_FAILED;
	} else {
		return RAS_IFACE_TECH_FAILED;
	}


	return RAS_IFACE_TECH_FAILED;
}


int	Ras_sqlite3_interface::load_swapped_tile(Ras_phtile *tile)
{
	sqlite3_stmt *linfo;

	int	rc, rce;
	
	stat_load++;
	
	char	*pom = sqlite3_mprintf("select cont from rg7 indexed by rg7_ind where x=%d and y=%d and pyr=%d;", tile->tile_xy.x, tile->tile_xy.y, tile->pyramid_level);
	

	//printf("PRik-load: %s\n", pom);
	
	rc = sqlite3_prepare(db, pom, -1, &linfo, 0);
	
	sqlite3_free(pom);
	
	if (rc == SQLITE_OK && linfo != NULL) {
		rce = sqlite3_step(linfo) == SQLITE_ROW;
		
		//printf("rce %d\n", rce);
		if (rce) {
			Compressed_ba cab((char *)sqlite3_column_blob(linfo,0));
			
			tile->read_from_compressed_ba(cab);
		}
		
		rc = sqlite3_finalize(linfo);
		
		return (rce!=0) ? RAS_IFACE_OK : RAS_IFACE_TECH_FAILED;
	} else {
		printf("Error read\n");
		return RAS_IFACE_TECH_FAILED;
	}

	return RAS_IFACE_TECH_FAILED;
}


int	Ras_sqlite3_interface::swapped_tile_available(Ras_phtile *tile)
{
	// file access statistics
	stat_test++;

	sqlite3_stmt *linfo;
	int	rce;
	
	// generate the SQL statement
	// STA checks existence of a record with the given xy coords
	char	*pom = sqlite3_mprintf("select x from rg7 indexed by rg7_ind where x=%d and y=%d and pyr=%d;", tile->tile_xy.x, tile->tile_xy.y, tile->pyramid_level);

	//printf("PRik-test: %s\n", pom);
	
	rce = sqlite3_prepare(db, pom, -1, &linfo, 0);
	
	sqlite3_free(pom);
	
	// sqlite3_prepare proceeded ok
	if (rce == SQLITE_OK && linfo != NULL) {
		
		rce = sqlite3_step(linfo);
		//printf("test-rce %d\n", rce);
		
		sqlite3_finalize(linfo);
		
		if (rce == SQLITE_OK || rce == SQLITE_DONE)
			return RAS_IFACE_OK_BUT_NO_DATA;
		
		if (rce == SQLITE_ROW)
			return RAS_IFACE_OK;
		
		return RAS_IFACE_TECH_FAILED;
	} else {
		//printf("COmpletely wrong\n");
		return RAS_IFACE_TECH_FAILED;
	}


	return RAS_IFACE_TECH_FAILED;
}


bool	Ras_sqlite3_interface::write_metadata()
{
	
	Region breg = get_base_region();
	ICoord bmin = get_minimum_region();

	//!!! check
	sqlite3_exec(db, "delete from metadata; ", 0, 0, 0);
	
	char	*cmd = sqlite3_mprintf("insert into metadata values (%d, %d, %f, %f, %f, %f, %d, %d, %d);", bmin.x, bmin.y, breg.lb.x, breg.lb.y, breg.rt.x, breg.rt.y, breg.countX(), breg.countY(), metad.null_value);
	
	//!!! check
	sqlite3_exec(db, cmd, 0, 0, 0);
	
	sqlite3_free(cmd);
	
	return true;
}


bool	Ras_sqlite3_interface::load_metadata()
{
	char *zErrMsg;

	// fetch the data and decode using the given call-back
	int rc = sqlite3_exec(db, "select * from metadata; ", metadata_callback, this, &zErrMsg);
	
	return rc == SQLITE_OK;
}


	
void	Ras_sqlite3_interface::transaction(bool begin)
{
	// open/close a DB transaction
	// SQLITE3 does not touch the DB journal during
	// particular insert/select/update
	
	if (begin)
		assert(sqlite3_exec(db, "BEGIN;", 0, 0, 0) == SQLITE_OK);
	else
		assert(sqlite3_exec(db, "END;", 0, 0, 0) == SQLITE_OK);
}


int metadata_callback(void *NotUsed, int argc, char **argv, char **azColName)
{
	int i;
	Ras_sqlite3_interface *th = (Ras_sqlite3_interface *)NotUsed;
   
	for(i=0; i<argc; i++){
		//printf("%s = %s\n", azColName[i], argv[i] ? argv[i] : "NULL");
		th->metad.setAttr(rstring(azColName[i]), rstring(argv[i]));
	}

	//printf("\n");
	return 0;
}
