#include "ras_tiling.h"

void	Ras_tiling::initialize()
{
}


ICoord	Ras_tiling::ptranslate(Coord coord)
{
	ICoord logi = metad->base_region.coords_to_point(coord);
	
	return ltranslate(logi);
}


ICoord	Ras_tiling::ltranslate(ICoord coord)
{
	ICoord mtr = metad->minimum_region;
	ICoord res(
		std::min(last_tile.x, coord.x / mtr.x), 
		std::min(last_tile.y, coord.y / mtr.y)
	);
	
	return res;
}


Region	Ras_tiling::region_of_a_tile_xy(ICoord tile_xy)
{
	Region res;
	Region &base = metad->base_region;
	
	// number of cells in [X,Y] dimensions
	Coord	mtsize = icoord_2_coord(metad->minimum_region);
	// size of a tile in geo-coordinates
	Coord	size_of_1tile = base.cell_size() * mtsize;
	
	// shift the LB of a given tile
	res.lb = metad->base_region.lb + (size_of_1tile*icoord_2_coord(tile_xy));
	
	ICoord	num_tiles = number_of_tiles();
	
	//std::cout << "Tile_xy " << tile_xy << "\n";
	//std::cout << "numoftiles " << num_tiles << "\n";
	
	// default assumption
	res.rt = res.lb + size_of_1tile;
	res.size = metad->minimum_region;
	
	int	base_count;
	
	// if the given tile is the last one in X dimension ...
	// (the tile has then a larger size...
	if (tile_xy.x == (num_tiles.x - 1)) {
		base_count = (num_tiles.x-1) * metad->minimum_region.x;
		res.size.x = metad->base_region.countX() - base_count;
		res.rt.x = metad->base_region.rt.x;
	}
	
	// last in Y-dimension
	if (tile_xy.y == (num_tiles.y - 1)) {
		base_count = (num_tiles.y-1) * metad->minimum_region.y;
		res.size.y = metad->base_region.countY() - base_count;
		res.rt.y = metad->base_region.rt.y;
	} 
	
	/*
	std::cout << "Region " << tile_xy << "\n";
	std::cout << "Sz " << res.size << "\n";
	std::cout << "Subr " << res << "\n";
	*/
	
	
	return res;
}


ICoord	Ras_tiling::ilb_of_a_tile_xy(ICoord tile_xy)
{
	return metad->minimum_region * tile_xy;
}


ICoord	Ras_tiling::comp_number_of_tiles()
{
	ICoord mtr = metad->minimum_region;
	ICoord br = metad->base_region.countXY();
	
	return ICoord(
		std::max(1, br.x / mtr.x), 
		std::max(1, br.y / mtr.y)
		);
}


size_t	Ras_tiling::comp_number_of_tiles_scalar()
{
	ICoord nt = comp_number_of_tiles();
	return nt.x * nt.y;
}


size_t	Ras_tiling::comp_largest_cellscount()
{
	ICoord sz = metad->minimum_region;
	count_type c1 = std::max(sz.x, sz.y);
	
	return 4*c1*c1;
}

