#ifndef	__RAS_TILING_H
#define	__RAS_TILING_H

#include "basics.h"
#include "ras_phtile.h"
#include "ras_metadata.h"

/*
	Class Ras_tiling defines the translation
	from Coord -> Tile_xy
	
	tile_xy=(0,0) is the LB tile.
	
	Most of the tiles are of metad.minimum_region size. 
	Depending on raster's size and minimum_region size, the most-right column of tiles and most-top row of tiles might be of larger size.
	
	Translation is done in constant time.
*/



struct	Ras_tiling {
	Ras_tiling(Ras_metadata *emet) : metad(emet) {
		tiles = comp_number_of_tiles();
		last_tile.x = std::max(0, tiles.x-1);
		last_tile.y = std::max(0, tiles.y-1);
	}
	
	// prepare for translations
	void	initialize();
	
	// translate a given coordinate (geographic/logic) to a tile_xy
	ICoord	ptranslate(Coord coord);
	ICoord	ltranslate(ICoord coord);
	
	// compute a region for a given tile_xy
	// assume that each tile may be of different size
	Region	region_of_a_tile_xy(ICoord tile_xy);
	ICoord	ilb_of_a_tile_xy(ICoord tile_xy);
	
	// compute the number of tiles in x/y direction
	ICoord	comp_number_of_tiles();
	ICoord	&number_of_tiles() { return tiles; }
	size_t	comp_number_of_tiles_scalar();
	
	size_t	comp_largest_cellscount();
	
	// reference to the file's metadata (stored in Ras_file_interface)
	Ras_metadata *metad;
	
	// 
	ICoord	tiles, last_tile;
};

#endif	/* __RAS_TILING_H */