#ifndef __RAS_WINDOW_H
#define __RAS_WINDOW_H

#include "ras_win_abstract.h"
#include "bmp/EasyBMP.h"

/*
	Basic raster window of given size [x,y]
	TODO: 
		1) optimized loads when shifted
		2) user comfortable R/W access (in abstract manner)
			-- spec for writing
*/

class Ras_window : Ras_win_abstraction {
public:
    // create a window of size "size" resampled by region "emon"
    Ras_window(Ras_phys_file &fl, Region emon, ICoord sz) : Ras_win_abstraction(fl, emon), size(sz) {
        wbuf = new General_2D_buffer(size);
	assert(wbuf != 0);
	currILB = ICoord(0,0);
    }
    
    // place LB of window at "crd"
    bool    point_at(ICoord crd);
    // shift
    bool    shift_with_direction(int dir_code, int n_cells=1);
    
    dtype   operator() (int x, int y);
    
//protected:
    bool    load_window_contents();
    
    void	export_bmp(rstring fn);
    
    // window's size 
    ICoord  size;
    // window's content (loaded via load_window_contents())
    General_2D_buffer *wbuf;
    // current LB corner of the window
    ICoord	currILB;
};

class   mBMP : public BMP {
public:
        void    SetPixel(int x, int y, RGBApixel pp) {  
                BMP::SetPixel(x, TellHeight()-y-1, pp);
        }
};

#endif /* __RAS_WINDOW_H */