#include "rg7_system.h"


RG7_system RG7;

RG7_system::RG7_system() 
{
	// global startup
	ras_init();
}


RG7_system::~RG7_system() 
{
	// close all registered open files (see RG7_open_file)
	close_all_files();
	
	// global shutdown
	ras_close();
}


RG_ID	RG7_system::def_empty_id()
{
	RG_ID	id;
	
	// TODO: put DB, loc and mapset default values
	
	return id;
}


RG_ID	RG7_system::def_id_with_name(rstring fn)
{
	RG_ID	id = def_empty_id();
	
	id.filename = fn;
	
	return id;
}


String_set RG7_system::g_list_rast(RG_ID *mapset)
{
    String_set res;
    
    if (mapset==NULL)
	    mapset = &active_mapset;
    
    // TODO: !!!!!!
    
    res.insert(rstring("f1"));
    res.insert(rstring("fgeol"));
    res.insert(rstring("fgeolbig"));
    
    return res;
}


bool    RG7_system::open(rstring name)
{
	if (!exists(name)) {
		std::cout << "File " << name << " does not exist\n";
		return false;
	}

	RG_ID rid = def_id_with_name(name);
	RG7_open_file *handle = is_file_registered(&rid);
    
	if (!handle) {
		handle = prim_open_file(&rid);
		if (!handle) 
			return false;
	}
    
	handle->access_counter++;
    
	return true;
}


bool    RG7_system::close(rstring name)
{
	RG_ID rid = def_id_with_name(name);
	RG7_open_file *file = is_file_registered(&rid);
    
	if (!file) return false;
    
	file->access_counter--;
	if (file->access_counter<=0) 
		return prim_close_file(file);
    
	return true;
}


bool    RG7_system::create(rstring name, Ras_metadata *metad)
{
	RG_ID	rid = def_id_with_name(name);
	RG7_open_file *file = is_file_registered(&rid);
    
	if (file) return false;
    
	file = prim_create_file(&rid, metad);
	if (!file) return false;
    
	file->access_counter++;
    
	return true;
}


Ras_phys_file *RG7_system::get_phys_file(RG_ID *rid)
{
	RG7_open_file *file = is_file_registered(rid);
	if (!file) return NULL;
	
	return file->rfile;
}


Ras_phys_file *RG7_system::get_phys_file(rstring bname)
{
	RG_ID id = def_id_with_name(bname);
	return get_phys_file(&id);
}


bool    RG7_system::exists(rstring name)
{
	// TODO: some sort of a connection to GRASS directory structures
	// TODO: define own RG7 directories in a mapset
	return true;
	//return g_list_rast().count(name)>0;
}


RG7_open_file *RG7_system::is_file_registered(RG_ID *rid)
{
	// use RG_ID::registration_name() as the key
	if (file_handles.count(rid->registration_name())>0) {
		return file_handles[rid->registration_name()];
	}
    
	return NULL;
}


Ras_file_interface *RG7_system::instance_an_interface_for(RG_ID *rid)
{
	// regarding the RG_ID, create an appropriate iface object
	switch (rid->file_iface_type()) {
		case RG_IFACE_RG7:
			return new Ras_sqlite3_interface(rid);
		default:
			return NULL;
	}
	
	return NULL;
}


RG7_open_file *RG7_system::prim_open_file(RG_ID *rid)
{
	// create a new system record
	RG7_open_file *file = new RG7_open_file(rid);
	
	// create a new interface object
	Ras_file_interface *iface = instance_an_interface_for(rid);
	if (iface == NULL) {
		std::cout << "prim_open_file internal error\n";
		return NULL;
	}
	
	// some error on opening
	if (!iface->open()) {
		delete iface;
		return NULL;
	}
    
	// create file's memory (tiles) manager
	file->rfile = new Ras_phys_file(*iface);
    
	// register it
	file_handles[file->id.registration_name()] = file;
    
	// return the handle
	return file;
}


RG7_open_file *RG7_system::prim_create_file(RG_ID *rid, Ras_metadata *metad)
{
	// create a new interface object
	Ras_file_interface *iface = instance_an_interface_for(rid);
	if (iface == NULL) {
		std::cout << "prim_create_file internal error\n";
		return NULL;
	}
	
	// copy the basic metadata settings
	// TODO: extend with some other details
	iface->metad.base_region = metad->base_region;
	iface->metad.minimum_region = metad->minimum_region;
    
	// first, do CREATE the storage
	if (!iface->create()) {
		delete iface;
		return NULL;
	}
    
	// when created, CLOSE it for further re-opening
	if (!iface->close()) {
		delete iface;
		return NULL;
	}
    
	// the iface is going to be instanced in the further "open" operation
	delete iface;
           
	// when created, do open it finally
	return prim_open_file(rid);
}


bool    RG7_system::prim_close_file(RG7_open_file *file)
{
	// obtain the iface (could be formulated better...)
	assert(file->rfile!=0);
	
	Ras_file_interface *iface = &file->rfile->iface;
	// delete the Ras_phys_file (!!! keep in mind that Phys_file does not delete the iface object)
	delete file->rfile;
	
	// delete the interface object
	delete iface;
    
	// deregister
	file_handles.erase(file->id.registration_name());
	delete file;
    
	// no error ???
	// TODO: check the proceeding of the above operations to handle some eventual errors
	return true;
}


void    RG7_system::close_all_files()
{
	RG7_open_file *fi;
	int err_cnt=0;
    
	// while there is a file registered
	while (file_handles.size()>0) {
		// take its handle
		fi = file_handles.begin()->second;
		// do close it
		if (!prim_close_file(fi)) 
			err_cnt++;
	}
    
	if (err_cnt)
		printf("!!! Close_all_files: %d errors\n", err_cnt);
}

bool	RG7_system::ras_init()
{
	time_t gt;
	time(&gt);
    
	init_genrand((unsigned long)gt);
	
	// TODO: initializace other interface methods
    
	//!!! TODO: catch eventual errors
	sqlite3_initialize();
    

	return true;
}


bool	RG7_system::ras_close()
{
    
	//!!! TODO: catch eventual errors
	sqlite3_shutdown();
    
	return true;
}
