#ifndef __RG7_SYSTEM
#define __RG7_SYSTEM

/*
	Local Raster Administrator
	
	No user right/permissions support at the moment.
	
*/

#include "basics.h"

#include "grass_raster_id.h"

#include "ras_phtile.h"
#include "ras_metadata.h"
#include "ras_file_interface.h"

#include "rg7_interfaces.h"

#include "ras_phys_file.h"

#include "ras_window.h"

// RG7_open_file describes the opened files.
// The structure is for system use ONLY!!! Not to be presented to users.
struct 	RG7_open_file {
	RG7_open_file(RG_ID *rid) : id(*rid) {
		access_counter = 0;
		rfile = NULL;
	}
	
	RG7_open_file(Ras_phys_file *erfile) : rfile(erfile) {
		access_counter = 0;
	}
    
	// identification of the file following the GRASS'es convention (database, location, mapset, filename)
	RG_ID	id;
	
	// file's memory manager
	Ras_phys_file *rfile;
	
	// how many mutual accesses (when drops to zero, file gets closed)
	int access_counter;
};

// map key: file's UNIQUE name (composed of its all components)
typedef std::map<rstring, RG7_open_file *> RG7_open_file_map;
typedef RG7_open_file_map::iterator RG7_open_file_map_i;

class RG7_system {
public:
	
    RG7_system();
    ~RG7_system();
    
    // operations over the rasters --- USER API
    String_set g_list_rast(RG_ID *mapset=NULL);
    
    // API for user's perspective (simple filename identifiers)
    bool    open(rstring name);
    bool    close(rstring name);
    bool    create(rstring name, Ras_metadata *metad);
    bool    exists(rstring name);
    
    // creating user cursors and windows -- USER API
    Ras_window	*win(rstring raster_fn, ICoord mon_size, ICoord sz_window);
    Ras_window	*win(rstring raster_fn, ICoord sz_window);
    Ras_window	*win(rstring raster_fn);
    Ras_window 	*win_primitive(Ras_phys_file *phf, Region region, ICoord sz_window);
    
    // for special purposes
    Ras_phys_file *get_phys_file(RG_ID *rid);
    Ras_phys_file *get_phys_file(rstring bname);
    
    // get the current monitor settings
    Region	monitor();
    void	set_monitor(rstring fn);
    // return an empty default ID (fills with DB, location and mapset components)
    RG_ID	def_empty_id();
    // add a filename 
    RG_ID	def_id_with_name(rstring fn);
    
  
protected:
    
	// General init/close operations (e.g. SQlite3 init)
    bool    ras_init();
    bool    ras_close();
    
    RG7_open_file *is_file_registered(RG_ID *rid);
    
    // primitive operations - open/create/close
    // they don't check correctness of their calls
    Ras_file_interface *instance_an_interface_for(RG_ID *rid);
    RG7_open_file *prim_open_file(RG_ID *rid);
    RG7_open_file *prim_create_file(RG_ID *rid, Ras_metadata *metad);
    bool    prim_close_file(RG7_open_file *file);
    
    // flush all caches, clear lists
    void    close_all_files();
    
    // ----------------- LOW priority internal threads
    // Multi-threading -- load tiles in AOI in advance
    void    thread_body_areas_of_interest();
    // ??? flushing the caches
    void    thread_body_fflush();	
    
    ///////////////////// DATA
    RG7_open_file_map file_handles;
    RG_ID	active_mapset;
    Region	current_monitor;
};

extern RG7_system RG7;

#endif  /* __RG7_SYSTEM */