#ifndef	__TILE_STORAGE_H
#define	__TILE_STORAGE_H

/*
	Tile_storage is a basic ADT (container) for registering Phtiles.
	
	In fact, it's a map:
	map[tile_xy] -> Phtile_map
	
	The ADT is currently implemented using STL map (accessed in log time).
	
	TODO: reimplement Tile_storage to achieve constant time access (some sort of hashed list).
*/

#include "basics.h"
#include "ras_phtile.h"

// define STL pair as a key <tile_xy.x, tile_xy.y>
typedef	std::pair<count_type, count_type> IntIdx;
IntIdx	makepair(ICoord coord);


// internal data type and its iterator
typedef	std::map<IntIdx, Ras_phtile *> Phtile_map;
typedef	Phtile_map::iterator Phtile_map_i;


struct	Tile_storage {

	Tile_storage() {
	}
	
	~Tile_storage() {
		// explicitly...
		clear();
	}
	
	// evident
	bool	exists(ICoord tile_xy);
	
	// get a record for the given key. Invokes "exists(tile_xy)" if sure_exists==false. If you previously checked tile's existence, save some time ommitting the further checking.
	Ras_phtile *get(ICoord tile_xy, bool sure_exists=false);
	
	// always checks the existence of pht->tile_xy key.
	bool	insert(Ras_phtile *pht);
	
	// similarly like "get"...
	bool	erase(Ras_phtile *pht, bool sure_exists=false);
	
	// clear the data structure
	void    clear() { map.clear(); }
	
	// number of records
	int	size() { return (int) map.size(); }
	
	// iterators stard/end
	Phtile_map_i begin() { return map.begin(); }
	Phtile_map_i end() { return map.end(); }
	
protected:
	Phtile_map map;
};

#endif	/* __TILE_STORAGE_H */